from __future__ import annotations
from collections import namedtuple
from dataclasses import dataclass
from ipymarkup.demo import format_span_line_markup
import json

from nlp_framework.containers.spans import Span, SpanGroup
from .data_displayer import Displayable

from ipymarkup import show_span_line_markup

class Document(Displayable):
    def __init__(self,text):
        self._text = text  
        self._entities = SpanGroup()
    def set_entities(self,ents):
        self._entities = ents
    @property
    def entities(self):
        return self._entities
    @property
    def text(self):
        return self._text
    @classmethod
    def read_json(cls,file_path):
        with open(file_path, 'r') as infile:
            data = json.load(infile)
            document = cls(data["doc_id"],data["text"])
        if "entities" in data:
            entities = SpanGroup.from_dicts(data["entities"])
            document.set_entities(entities)
        return document
    def write_json(self,file_path):
        data = self.get_data()
        with open(file_path, 'w+') as infile:
            json.dump(data,infile)
    def get_data(self):
        if self._entities:
            entities = self._entities.data
        else:
            entities = None
        return {"text":self.text,"entities":entities}
    def __repr__(self):
        if self._entities:
            entities_repr = self._entities.__repr__()
        else:
            entities_repr = None
        return f"message:{self.message}\n{'-'*20}\nentities:{entities_repr}"
    def segment(self:Document,tokenizer,max_span_per_new_doc) -> list(Document):
        assert not self.check_for_span_overlaps()
        def segment_recursive(remaining_text,remaining_spans,doc_number=0,prev_seq_len=0):
            # new_doc_id = self.doc_id + "_" + str(doc_number)
            omitted_spans = 0
            new_document_spans = []
            spans_added = 0 
            segment_text = ""
            while spans_added < len(remaining_spans):#we've processed all remaining spans
                span_to_add =  remaining_spans[spans_added]
                span_start_index_in_new_doc = span_to_add.start_index - prev_seq_len
                span_end_index_in_new_doc = span_to_add.end_index - prev_seq_len
                tentative_segment_text = remaining_text[:span_end_index_in_new_doc]
                tokens = tokenizer.tokenize(tentative_segment_text)
                if (len(new_document_spans) < max_span_per_new_doc and len(['[CLS]'] + tokens + ['[SEP]']) < 256):
                    segment_text = tentative_segment_text
                    #new_span_id = new_doc_id + str((len(new_document_spans) + omitted_spans) % max_span_per_new_doc)
                    if span_start_index_in_new_doc < span_end_index_in_new_doc  and span_start_index_in_new_doc>= 0 and span_end_index_in_new_doc > 0:
                        new_document_spans.append(Span(span_start_index_in_new_doc,span_end_index_in_new_doc,span_to_add.concept_id))
                    else:
                        omitted_spans += 1
                    spans_added += 1
                else:
                    segmented_doc = Document(segment_text)
                    segmented_doc.set_entities(new_document_spans)
                    remaining_segmented_docs,remaining_omitted_spans = segment_recursive(remaining_text[len(segment_text):],remaining_spans[spans_added:],doc_number+1,prev_seq_len+len(segment_text))
                    return [segmented_doc] + remaining_segmented_docs, omitted_spans + remaining_omitted_spans
            segmented_doc = Document(remaining_text,new_document_spans)
            return [segmented_doc],omitted_spans#base case
        segmented_docs,omitted_spans = segment_recursive(self.message,self.spans)
        print(f"spans Omitted: {omitted_spans}")
        return segmented_docs

    # def produce_html(self,lkb=None):
    #     if lkb == None:
    #         spans = [(span.start_index,span.end_index,span.concept_id) for span in self._spans]
    #     else:
    #         spans = [(span.start_index,span.end_index,span.concept_name) for span in self._spans]
    #     html = "".join(format_span_line_markup(self.text, spans))
    #     return html
