class StringStore:
    """Look up strings by 64-bit hashes.
    DOCS: https://spacy.io/api/stringstore
    """
    def __init__(self, strings):
        """Create the StringStore.
        strings (iterable): A sequence of unicode strings to add to the store.
        """
        self._string_index_array = strings
        self._string_to_index_dict = {}
        if strings is not None:
            for string in strings:
                self.add(string)
    def add(self, string):
        """Add a string to the StringStore.
        string (str): The string to add.
        RETURNS None
        """
        if string not in self._string_to_index_dict:
            self._string_to_index_dict[string] = len(self._string_index_array)
        
    def __getitem__(self, string_or_id):
        """Retrieve a string from a given index, or vice versa.
        string_or_id (str / int): The value to encode.
        Returns (str / int): The value to be retrieved.
        """
        if type(string_or_id) == str:
            if string_or_id in self._string_to_index_dict:
                return self._string_to_index_dict[string_or_id]
            else:
                raise ValueError
        elif type(string_or_id) == int:
            if string_or_id < len(self._string_index_array):
                return self._string_index_array[string_or_id]
            else:
                raise ValueError
        else:
            raise TypeError
            
        
